Fix adu link.
[osl.git] / osl.c
diff --git a/osl.c b/osl.c
index 1179307c1d21ddf0c3d9bb0731ea1fcb3cc700c8..67160d2b3bb14539edf1cd5f8b886d55940d0f63 100644 (file)
--- a/osl.c
+++ b/osl.c
@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2008 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2007-2009 Andre Noll <maan@systemlinux.org>
  *
  * Licensed under the GPL v2. For licencing details see COPYING.
  */
@@ -10,7 +10,6 @@
 
 #include "log.h"
 #include "osl.h"
-#include "error.h"
 #include "util.h"
 #include "osl_core.h"
 
@@ -38,8 +37,6 @@ static const unsigned int errmsgidx[] = {
 
 __export const char *osl_strerror(int num)
 {
-       if (IS_SYSTEM_ERROR(num))
-               return strerror((num) & ((1 << SYSTEM_ERROR_BIT) - 1));
        return msgstr.str + errmsgidx[num];
 }
 
@@ -64,20 +61,18 @@ static void __attribute ((constructor)) init_loglevel(void)
 __printf_2_3 void __log(int ll, const char* fmt,...)
 {
        va_list argp;
-       FILE *outfd;
        struct tm *tm;
        time_t t1;
        char str[255] = "";
 
        if (ll < loglevel)
                return;
-       outfd = stderr;
        time(&t1);
        tm = localtime(&t1);
        strftime(str, sizeof(str), "%b %d %H:%M:%S", tm);
-       fprintf(outfd, "%s ", str);
+       fprintf(stderr, "%s ", str);
        va_start(argp, fmt);
-       vfprintf(outfd, fmt, argp);
+       vfprintf(stderr, fmt, argp);
        va_end(argp);
 }
 
@@ -181,7 +176,7 @@ static int disk_storage_name_of_row(const struct osl_table *t,
        *name = disk_storage_name_of_object(t, &obj);
        if (*name)
                return 1;
-       return -ERRNO_TO_ERROR(ENOMEM);
+       return -E_OSL_NOMEM;
 }
 
 static void column_name_hash(const char *col_name, HASH_TYPE *hash)
@@ -250,7 +245,7 @@ int init_table_structure(const struct osl_table_description *desc,
 {
        const struct osl_column_description *cd;
        struct osl_table *t = calloc(1, sizeof(*t));
-       int i, ret = -ERRNO_TO_ERROR(ENOMEM), have_disk_storage_name_column = 0;
+       int i, ret = -E_OSL_NOMEM, have_disk_storage_name_column = 0;
 
        if (!t)
                return ret;
@@ -265,7 +260,7 @@ int init_table_structure(const struct osl_table_description *desc,
        ret = -E_OSL_NO_COLUMNS;
        if (!desc->num_columns)
                goto err;
-       ret = -ERRNO_TO_ERROR(ENOMEM);
+       ret = -E_OSL_NOMEM;
        t->columns = calloc(desc->num_columns, sizeof(struct osl_column));
        if (!t->columns)
                goto err;
@@ -334,8 +329,8 @@ err:
  * \param map The memory mapping of the index file.
  * \param desc The values found in the index header are returned here.
  *
* Read the index header, check for the paraslash magic string and the table version number.
* Read all information stored in the index header into \a desc.
  *
  * \return Standard.
  *
@@ -344,7 +339,7 @@ err:
 int read_table_desc(struct osl_object *map, struct osl_table_description *desc)
 {
        char *buf = map->data;
-       uint8_t version;
+       uint8_t version, compat_version, create_version;
        uint16_t header_size;
        int ret, i;
        unsigned offset;
@@ -352,10 +347,22 @@ int read_table_desc(struct osl_object *map, struct osl_table_description *desc)
 
        if (map->size < MIN_INDEX_HEADER_SIZE(1))
                return -E_OSL_SHORT_TABLE;
-       if (strncmp(buf + IDX_PARA_MAGIC, PARA_MAGIC, strlen(PARA_MAGIC)))
+       if (strncmp(buf + IDX_OSL_MAGIC, OSL_MAGIC, strlen(OSL_MAGIC)))
                return -E_OSL_NO_MAGIC;
        version = read_u8(buf + IDX_VERSION);
-       if (version < MIN_TABLE_VERSION || version > MAX_TABLE_VERSION)
+       /*
+        * The on-disk version consists of two version numbers: the
+        * create_version (low 4 bits) is the CURRENT_TABLE_VERSION version
+        * number of the library that created the table, and compat_version
+        * (high 4 bits) tells us the lowest version of the library that can
+        * still read this table.
+        */
+       create_version = version & 0xf;
+       compat_version = version >> 4;
+       INFO_LOG("create_version: %u, compat_version: %u\n", create_version,
+               compat_version);
+       if (create_version < MIN_TABLE_VERSION /* table too old */
+               || compat_version > CURRENT_TABLE_VERSION) /* libosl too old */
                return -E_OSL_VERSION_MISMATCH;
        desc->flags = read_u8(buf + IDX_TABLE_FLAGS);
        desc->num_columns = read_u16(buf + IDX_NUM_COLUMNS);
@@ -368,7 +375,7 @@ int read_table_desc(struct osl_object *map, struct osl_table_description *desc)
        desc->column_descriptions = calloc(desc->num_columns,
                sizeof(struct osl_column_description));
        if (!desc->column_descriptions)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        offset = IDX_COLUMN_DESCRIPTIONS;
        FOR_EACH_COLUMN(i, desc, cd) {
                char *null_byte;
@@ -388,7 +395,7 @@ int read_table_desc(struct osl_object *map, struct osl_table_description *desc)
                ret = -E_OSL_INDEX_CORRUPTION;
                if (!null_byte)
                        goto err;
-               ret = -ERRNO_TO_ERROR(ENOMEM);
+               ret = -E_OSL_NOMEM;
                cd->name = strdup(buf + offset + IDX_CD_NAME);
                if (!cd->name)
                        goto err;
@@ -431,7 +438,7 @@ static int compare_table_descriptions(struct osl_table *t)
                struct osl_column_description *cd;
                unsigned diff = t->desc->num_columns - desc.num_columns;
                INFO_LOG("extending table by %u volatile columns\n", diff);
-               ret = -ERRNO_TO_ERROR(ENOMEM);
+               ret = -E_OSL_NOMEM;
                desc.column_descriptions = realloc(desc.column_descriptions,
                        t->desc->num_columns * sizeof(struct osl_column_description));
                if (!desc.column_descriptions)
@@ -486,11 +493,12 @@ static int create_table_index(struct osl_table *t)
                t->desc->name);
        buf = calloc(1, size);
        if (!buf)
-               return -ERRNO_TO_ERROR(ENOMEM);
-       sprintf(buf + IDX_PARA_MAGIC, "%s", PARA_MAGIC);
+               return -E_OSL_NOMEM;
+       sprintf(buf + IDX_OSL_MAGIC, "%s", OSL_MAGIC);
        write_u8(buf + IDX_TABLE_FLAGS, t->desc->flags);
        write_u8(buf + IDX_DIRTY_FLAG, 0);
-       write_u8(buf + IDX_VERSION, CURRENT_TABLE_VERSION);
+       write_u8(buf + IDX_VERSION, CURRENT_TABLE_VERSION
+               + (COMPAT_TABLE_VERSION << 4));
        write_u16(buf + IDX_NUM_COLUMNS, t->num_mapped_columns + t->num_disk_storage_columns);
        write_u16(buf + IDX_HEADER_SIZE, t->index_header_size);
        offset = IDX_COLUMN_DESCRIPTIONS;
@@ -513,7 +521,7 @@ static int create_table_index(struct osl_table *t)
        if (filename)
                ret = write_file(filename, buf, size);
        else
-               ret = -ERRNO_TO_ERROR(ENOMEM);
+               ret = -E_OSL_NOMEM;
        free(buf);
        free(filename);
        return ret;
@@ -534,18 +542,18 @@ __export int osl_create_table(const struct osl_table_description *desc)
                        continue;
                if (!table_dir) {
                        ret = osl_mkdir(desc->dir, 0777);
-                       if (ret < 0 && !is_errno(-ret, EEXIST))
+                       if (ret < 0 && ret != -E_OSL_DIR_EXISTS)
                                goto out;
                        table_dir = make_message("%s/%s", desc->dir,
                                desc->name);
-                       ret = -ERRNO_TO_ERROR(ENOMEM);
+                       ret = -E_OSL_NOMEM;
                        if (!table_dir)
                                goto out;
                        ret = osl_mkdir(table_dir, 0777);
                        if (ret < 0)
                                goto out;
                }
-               ret = -ERRNO_TO_ERROR(ENOMEM);
+               ret = -E_OSL_NOMEM;
                filename = column_filename(t, i);
                if (!filename)
                        goto out;
@@ -649,7 +657,7 @@ static int map_column(struct osl_table *t, unsigned col_num)
        int ret;
 
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        ret = osl_stat(filename, &statbuf);
        if (ret < 0) {
                free(filename);
@@ -690,7 +698,7 @@ int map_table(struct osl_table *t, enum map_table_flags flags)
                return -E_OSL_ALREADY_MAPPED;
        filename = index_filename(t->desc);
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        INFO_LOG("mapping table '%s' (index: %s)\n", t->desc->name, filename);
        ret = mmap_full_file(filename, flags & MAP_TBL_FL_MAP_RDONLY?
                O_RDONLY : O_RDWR, &t->index_map.data, &t->index_map.size, NULL);
@@ -765,6 +773,10 @@ int get_mapped_object(const struct osl_table *t, unsigned col_num,
        return 1;
 }
 
+/*
+ * It's OK to call this with result = rb_node = NULL.  If result is not NULL,
+ * and rb key was not found, result points to the parent node.
+ */
 static int search_rbtree(const struct osl_object *obj,
                const struct osl_table *t, unsigned col_num,
                struct rb_node **result, struct rb_node ***rb_link)
@@ -860,7 +872,7 @@ static int add_row_to_rbtrees(struct osl_table *t, uint32_t row_num,
        const struct osl_column_description *cd;
 
        if (!row)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        row->num = row_num;
        row->volatile_objects = volatile_objs;
        FOR_EACH_RBTREE_COLUMN(i, t, cd) {
@@ -1033,6 +1045,7 @@ int init_rbtrees(struct osl_table *t)
        /* add valid rows to rbtrees */
        t->num_invalid_rows = 0;
        for (i = 0; i < t->num_rows; i++) {
+               struct osl_object *volatile_objs;
                ret = row_is_invalid(t, i);
                if (ret < 0)
                        return ret;
@@ -1040,7 +1053,14 @@ int init_rbtrees(struct osl_table *t)
                        t->num_invalid_rows++;
                        continue;
                }
-               ret = add_row_to_rbtrees(t, i, NULL, NULL);
+               if (t->num_volatile_columns > 0) {
+                       volatile_objs = calloc(t->num_volatile_columns,
+                               sizeof(struct osl_object));
+                       if (!volatile_objs)
+                               return -E_OSL_NOMEM;
+               } else
+                       volatile_objs = NULL;
+               ret = add_row_to_rbtrees(t, i, volatile_objs, NULL);
                if (ret < 0)
                        return ret;
        }
@@ -1062,17 +1082,15 @@ __export int osl_open_table(const struct osl_table_description *table_desc,
                struct stat statbuf;
                char *dirname = column_filename(t, i);
 
-               ret = -ERRNO_TO_ERROR(ENOMEM);
+               ret = -E_OSL_NOMEM;
                if (!dirname)
                        goto err;
                /* check if directory exists */
-               ret = stat(dirname, &statbuf);
+               ret = osl_stat(dirname, &statbuf);
                free(dirname);
-               if (ret < 0) {
-                       ret = -ERRNO_TO_ERROR(errno);
+               if (ret < 0)
                        goto err;
-               }
-               ret = -ERRNO_TO_ERROR(ENOTDIR);
+               ret = -E_OSL_NOTDIR;
                if (!S_ISDIR(statbuf.st_mode))
                        goto err;
        }
@@ -1104,10 +1122,10 @@ static int create_disk_storage_object_dir(const struct osl_table *t,
                return 1;
        dirname = disk_storage_dirname(t, col_num, ds_name);
        if (!dirname)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        ret = osl_mkdir(dirname, 0777);
        free(dirname);
-       if (ret < 0 && !is_errno(-ret, EEXIST))
+       if (ret < 0 && ret != -E_OSL_DIR_EXISTS)
                return ret;
        return 1;
 }
@@ -1123,7 +1141,7 @@ static int write_disk_storage_file(const struct osl_table *t, unsigned col_num,
                return ret;
        filename = disk_storage_path(t, col_num, ds_name);
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        ret = write_file(filename, obj->data, obj->size);
        free(filename);
        return ret;
@@ -1136,7 +1154,7 @@ static int append_map_file(const struct osl_table *t, unsigned col_num,
        int ret;
 
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        ret = append_file(filename, obj->data, obj->size, new_size);
        free(filename);
        return ret;
@@ -1151,7 +1169,7 @@ static int append_row_index(const struct osl_table *t, char *row_index)
                return 1;
        filename = index_filename(t->desc);
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        ret = append_file(filename, row_index, t->row_index_size, NULL);
        free(filename);
        return ret;
@@ -1164,7 +1182,7 @@ static int truncate_mapped_file(const struct osl_table *t, unsigned col_num,
        char *filename = column_filename(t, col_num);
 
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        ret = truncate_file(filename, size);
        free(filename);
        return ret;
@@ -1174,20 +1192,20 @@ static int delete_disk_storage_file(const struct osl_table *t, unsigned col_num,
                const char *ds_name)
 {
        char *dirname, *filename = disk_storage_path(t, col_num, ds_name);
-       int ret, err;
+       int ret = 1;
 
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
-       ret = unlink(filename);
-       err = errno;
+               return -E_OSL_NOMEM;
+       if (unlink(filename) < 0)
+               ret = errno == ENOENT? -E_OSL_NOENT : -E_OSL_UNLINK;
        free(filename);
        if (ret < 0)
-               return -ERRNO_TO_ERROR(err);
+               return ret;
        if (!(t->desc->flags & OSL_LARGE_TABLE))
                return 1;
        dirname = disk_storage_dirname(t, col_num, ds_name);
        if (!dirname)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        rmdir(dirname);
        free(dirname);
        return 1;
@@ -1207,18 +1225,18 @@ __export int osl_add_and_get_row(struct osl_table *t, struct osl_object *objects
                return -E_OSL_BAD_TABLE;
        rb_parents = malloc(t->num_rbtrees * sizeof(struct rn_node*));
        if (!rb_parents)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        rb_links = malloc(t->num_rbtrees * sizeof(struct rn_node**));
        if (!rb_links) {
                free(rb_parents);
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        }
        if (t->num_mapped_columns) {
                new_row_index = malloc(t->row_index_size);
                if (!new_row_index) {
                        free(rb_links);
                        free(rb_parents);
-                       return -ERRNO_TO_ERROR(ENOMEM);
+                       return -E_OSL_NOMEM;
                }
        }
        /* pass 1: sanity checks */
@@ -1254,7 +1272,7 @@ __export int osl_add_and_get_row(struct osl_table *t, struct osl_object *objects
        if (t->num_disk_storage_columns) {
                ds_name = disk_storage_name_of_object(t,
                        &objects[t->disk_storage_name_column]);
-               ret = -ERRNO_TO_ERROR(ENOMEM);
+               ret = -E_OSL_NOMEM;
                if (!ds_name)
                        goto out;
        }
@@ -1297,7 +1315,7 @@ __export int osl_add_and_get_row(struct osl_table *t, struct osl_object *objects
        }
        /* pass 3: add entry to rbtrees */
        if (t->num_volatile_columns) {
-               ret = -ERRNO_TO_ERROR(ENOMEM);
+               ret = -E_OSL_NOMEM;
                volatile_objs = calloc(t->num_volatile_columns,
                        sizeof(struct osl_object));
                if (!volatile_objs)
@@ -1501,7 +1519,7 @@ static int rename_disk_storage_objects(struct osl_table *t,
                return 1; /* object did not change */
        old_ds_name = disk_storage_name_of_object(t, old_obj);
        new_ds_name = disk_storage_name_of_object(t, new_obj);
-       ret = -ERRNO_TO_ERROR(ENOMEM);
+       ret = -E_OSL_NOMEM;
        if (!old_ds_name || ! new_ds_name)
                goto out;
 
@@ -1513,7 +1531,7 @@ static int rename_disk_storage_objects(struct osl_table *t,
                old_filename = disk_storage_path(t, i, old_ds_name);
                new_filename = disk_storage_path(t, i, new_ds_name);
                if (!old_filename || !new_filename)
-                       ret = -ERRNO_TO_ERROR(ENOMEM);
+                       ret = -E_OSL_NOMEM;
                else
                        ret = osl_rename(old_filename, new_filename);
                free(old_filename);
@@ -1559,7 +1577,7 @@ __export int osl_update_object(struct osl_table *t, const struct osl_row *r,
                if (ret < 0)
                        return ret;
                ret = delete_disk_storage_file(t, col_num, ds_name);
-               if (ret < 0 && !is_errno(-ret, ENOENT)) {
+               if (ret < 0 && ret != -E_OSL_NOENT) {
                        free(ds_name);
                        return ret;
                }
@@ -1629,7 +1647,7 @@ __export int osl_open_disk_object(const struct osl_table *t, const struct osl_ro
        filename = disk_storage_path(t, col_num, ds_name);
        free(ds_name);
        if (!filename)
-               return -ERRNO_TO_ERROR(ENOMEM);
+               return -E_OSL_NOMEM;
        DEBUG_LOG("filename: %s\n", filename);
        ret = mmap_full_file(filename, O_RDONLY, &obj->data, &obj->size, NULL);
        free(filename);
@@ -334,8 +329,8 @@ err:
  * \param desc The values found in the index header are returned here.
  *
- * Read the index header, check for the paraslash magic string and the table version number.
- * Read all information stored in the index header into \a desc.
+ * Read the index header, check for the osl magic string and the table version
+ * number.  Read all information stored in the index header into \a desc.
  *
  * \return Standard.